from pyironscales.endpoints.base.base_endpoint import IronscalesEndpoint
from pyironscales.interfaces import (
    IGettable,
    IPuttable,
    IDeleteable,
)
from pyironscales.models.ironscales import PartnerCompany
from pyironscales.endpoints.ironscales.CompanyId911EmailEndpoint import CompanyId911EmailEndpoint
from pyironscales.endpoints.ironscales.CompanyIdAutoSyncEndpoint import CompanyIdAutoSyncEndpoint
from pyironscales.endpoints.ironscales.CompanyIdFeaturesEndpoint import CompanyIdFeaturesEndpoint
from pyironscales.endpoints.ironscales.CompanyIdManifestEndpoint import CompanyIdManifestEndpoint
from pyironscales.endpoints.ironscales.CompanyIdStatsEndpoint import CompanyIdStatsEndpoint
from pyironscales.types import (
    JSON,
    IronscalesRequestParams,
)


class CompanyIdEndpoint(
    IronscalesEndpoint,
    IGettable[PartnerCompany, IronscalesRequestParams],
    IPuttable[PartnerCompany, IronscalesRequestParams],
    IDeleteable[IronscalesRequestParams],
):
    def __init__(self, client, parent_endpoint=None) -> None:
        IronscalesEndpoint.__init__(self, client, "{id}", parent_endpoint=parent_endpoint)
        IGettable.__init__(self, PartnerCompany)
        IPuttable.__init__(self, PartnerCompany)
        IDeleteable.__init__(self, PartnerCompany)
        self._911_email = self._register_child_endpoint(CompanyId911EmailEndpoint(client, parent_endpoint=self))
        self.auto_sync = self._register_child_endpoint(CompanyIdAutoSyncEndpoint(client, parent_endpoint=self))
        self.features = self._register_child_endpoint(CompanyIdFeaturesEndpoint(client, parent_endpoint=self))
        self.manifest = self._register_child_endpoint(CompanyIdManifestEndpoint(client, parent_endpoint=self))
        self.stats = self._register_child_endpoint(CompanyIdStatsEndpoint(client, parent_endpoint=self))

    def get(
        self,
        data: JSON | None = None,
        params: IronscalesRequestParams | None = None,
    ) -> PartnerCompany:
        """
        Performs a GET request against the /company/{id} endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            AuthInformation: The parsed response data.
        """
        return self._parse_one(
            PartnerCompany,
            super()._make_request("GET", data=data, params=params).json(),
        )

    def put(
        self,
        data: JSON | None = None,
        params: IronscalesRequestParams | None = None,
    ) -> PartnerCompany:
        """
        Performs a PUT request against the /company/{id} endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            PartnerCompany: The parsed response data.
        """
        return self._parse_one(
            PartnerCompany,
            super()._make_request("PUT", data=data, params=params).json(),
        )

    def delete(self, data: JSON | None = None, params: IronscalesRequestParams | None = None) -> PartnerCompany:
        """
        Performs a DELETE request against the /company/{id}/ endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            PartnerCompany: The parsed response data.
        """
        return self._parse_one(PartnerCompany, super()._make_request("DELETE", data=data, params=params).json())



from pyironscales.endpoints.base.base_endpoint import IronscalesEndpoint
from pyironscales.endpoints.ironscales.IncidentIdStatsRemediationStatusesEndpoint import IncidentIdStatsRemediationStatusesEndpoint


class IncidentIdStatsEndpoint(
    IronscalesEndpoint,
):
    def __init__(self, client, parent_endpoint=None) -> None:
        IronscalesEndpoint.__init__(self, client, "stats", parent_endpoint=parent_endpoint)
        self.remediation_statuses = self._register_child_endpoint(IncidentIdStatsRemediationStatusesEndpoint(client, parent_endpoint=self))



from pyironscales.endpoints.base.base_endpoint import IronscalesEndpoint
from pyironscales.endpoints.ironscales.CompanyIdEndpoint import CompanyIdEndpoint
from pyironscales.endpoints.ironscales.CompanyCreateEndpoint import CompanyCreateEndpoint
from pyironscales.endpoints.ironscales.CompanyListEndpoint import CompanyListEndpoint


class CompanyEndpoint(
    IronscalesEndpoint,
):
    def __init__(self, client, parent_endpoint=None) -> None:
        IronscalesEndpoint.__init__(self, client, "company", parent_endpoint=parent_endpoint)
        self.create = self._register_child_endpoint(CompanyCreateEndpoint(client, parent_endpoint=self))
        self.list = self._register_child_endpoint(CompanyListEndpoint(client, parent_endpoint=self))

    def id(self, id: int) -> CompanyIdEndpoint:
        """
        Sets the ID for this endpoint and returns an initialized CompanyIdEndpoint object to move down the chain.

        Parameters:
            id (int): The ID to set.
        Returns:
            CompanyIdEndpoint: The initialized CompanyIdEndpoint object.
        """
        child = CompanyIdEndpoint(self.client, parent_endpoint=self)
        child._id = id
        return child



import typing
import json
from datetime import datetime, timedelta, timezone

from pyironscales.clients.base_client import IronscalesClient
from pyironscales.config import Config

if typing.TYPE_CHECKING:
    from pyironscales.endpoints.ironscales.CampaignsEndpoint import CampaignsEndpoint
    from pyironscales.endpoints.ironscales.CompanyEndpoint import CompanyEndpoint
    from pyironscales.endpoints.ironscales.EmailsEndpoint import EmailsEndpoint
    from pyironscales.endpoints.ironscales.IncidentEndpoint import IncidentEndpoint
    from pyironscales.endpoints.ironscales.IntegrationsEndpoint import IntegrationsEndpoint
    from pyironscales.endpoints.ironscales.MailboxesEndpoint import MailboxesEndpoint
    from pyironscales.endpoints.ironscales.MitigationEndpoint import MitigationEndpoint
    from pyironscales.endpoints.ironscales.PlanDetailsEndpoint import PlanDetailsEndpoint
    from pyironscales.endpoints.ironscales.SettingsEndpoint import SettingsEndpoint


class IronscalesAPIClient(IronscalesClient):
    """
    Ironscales API client. Handles the connection to the Ironscales API
    and the configuration of all the available endpoints.
    """

    def __init__(
        self,
        privatekey: str,
        scopes: list,
    ) -> None:
        """
        Initializes the client with the given credentials.

        Parameters:
            privatekey (str): Your Ironscales API private key.
        """
        self.privatekey: str = privatekey
        self.scopes: list = json.loads(scopes) if isinstance(json.loads(scopes), list) else [json.loads(scopes)]
        self.token_expiry_time: datetime = datetime.now(tz=timezone.utc)

        # Grab first access token
        self.access_token: str = self._get_access_token()

    # Initializing endpoints
    @property
    def campaigns(self) -> "CampaignsEndpoint":
        from pyironscales.endpoints.ironscales.CampaignsEndpoint import CampaignsEndpoint

        return CampaignsEndpoint(self)

    @property
    def company(self) -> "CompanyEndpoint":
        from pyironscales.endpoints.ironscales.CompanyEndpoint import CompanyEndpoint

        return CompanyEndpoint(self)

    @property
    def emails(self) -> "EmailsEndpoint":
        from pyironscales.endpoints.ironscales.EmailsEndpoint import EmailsEndpoint

        return EmailsEndpoint(self)

    @property
    def incident(self) -> "IncidentEndpoint":
        from pyironscales.endpoints.ironscales.IncidentEndpoint import IncidentEndpoint

        return IncidentEndpoint(self)

    @property
    def integrations(self) -> "IntegrationsEndpoint":
        from pyironscales.endpoints.ironscales.IntegrationsEndpoint import IntegrationsEndpoint

        return IntegrationsEndpoint(self)

    @property
    def mailboxes(self) -> "MailboxesEndpoint":
        from pyironscales.endpoints.ironscales.MailboxesEndpoint import MailboxesEndpoint

        return MailboxesEndpoint(self)

    @property
    def mitigation(self) -> "MitigationEndpoint":
        from pyironscales.endpoints.ironscales.MitigationEndpoint import MitigationEndpoint

        return MitigationEndpoint(self)

    @property
    def plan_details(self) -> "PlanDetailsEndpoint":
        from pyironscales.endpoints.ironscales.PlanDetailsEndpoint import PlanDetailsEndpoint

        return PlanDetailsEndpoint(self)

    @property
    def settings(self) -> "SettingsEndpoint":
        from pyironscales.endpoints.ironscales.SettingsEndpoint import SettingsEndpoint

        return SettingsEndpoint(self)

    def _get_url(self) -> str:
        """
        Generates and returns the URL for the Ironscales API endpoints based on the company url and codebase.
        Logs in an obtains an access token.
        Returns:
            str: API URL.
        """
        return f"https://appapi.ironscales.com/appapi"

    def _get_access_token(self) -> str:
        """
        Performs a request to the ConnectWise Automate API to obtain an access token.
        """
        auth_response = self._make_request(
            "POST",
            f"{self._get_url()}/get-token/",
            data={
            "key": self.privatekey,
            "scopes": self.scopes
                },
            headers={
                "Content-Type": "application/json",
                },
        )
        auth_resp_json = auth_response.json()
        token = auth_resp_json["jwt"]
        expires_in_sec = 43200
        self.token_expiry_time = datetime.now(tz=timezone.utc) + timedelta(seconds=expires_in_sec)
        return token

    def _refresh_access_token_if_necessary(self):
        if datetime.now(tz=timezone.utc) > self.token_expiry_time:
            self.access_token = self._get_access_token()

    def _get_headers(self) -> dict[str, str]:
        """
        Generates and returns the headers required for making API requests. The access token is refreshed if necessary before returning.

        Returns:
            dict[str, str]: Dictionary of headers including Content-Type, Client ID, and Authorization.
        """
        return {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "Authorization": f"Bearer {self.access_token}",
        }



from pyironscales.endpoints.base.base_endpoint import IronscalesEndpoint
from pyironscales.endpoints.ironscales.MitigationsIdDetailsEndpoint import MitigationsIdDetailsEndpoint
from pyironscales.endpoints.ironscales.MitigationsIdImpersonationEndpoint import MitigationsIdImpersonationEndpoint
from pyironscales.endpoints.ironscales.MitigationsIdIncidentsEndpoint import MitigationsIdIncidentsEndpoint
from pyironscales.endpoints.ironscales.MitigationsIdStatsEndpoint import MitigationsIdStatsEndpoint

class MitigationIdEndpoint(
    IronscalesEndpoint,
):
    def __init__(self, client, parent_endpoint=None) -> None:
        IronscalesEndpoint.__init__(self, client, "{id}", parent_endpoint=parent_endpoint)
        self.details = self._register_child_endpoint(MitigationsIdDetailsEndpoint(client, parent_endpoint=self))
        self.impersonation = self._register_child_endpoint(MitigationsIdImpersonationEndpoint(client, parent_endpoint=self))
        self.incidents = self._register_child_endpoint(MitigationsIdIncidentsEndpoint(client, parent_endpoint=self))
        self.stats = self._register_child_endpoint(MitigationsIdStatsEndpoint(client, parent_endpoint=self))



from __future__ import annotations

from datetime import datetime
from typing import Any

from pydantic import Field

from pyironscales.models.base.base_model import IronscalesModel

class Pagination(IronscalesModel):
    current_page: int | None = Field(default=None, alias="CurrentPage")
    current_page_count: int | None = Field(default=None, alias="CurrentPageCount")
    limit: int | None = Field(default=None, alias="Limit")
    total_count: int | None = Field(default=None, alias="TotalCount")
    next_page: int | None = Field(default=None, alias="NextPage")
    next_page_url: str | None = Field(default=None, alias="NextPageURL")
    next_page_token: str | None = Field(default=None, alias="NextPageToken")
    
class Campaigns(IronscalesModel):
    campaignID: int | None = Field(default=None, alias="CampaignId")
    campaignName: str | None = Field(default=None, alias="CampaignName")
    campaignStatus: str | None = Field(default=None, alias="CampaignStatus")
    flowType: str | None = Field(default=None, alias="FlowType")
    language: str | None = Field(default=None, alias="Language")
    maxEmailPerDay: int | None = Field(default=None, alias="MaxEmailPerDay")
    endDate: datetime | None = Field(default=None, alias="Modified")
    launchDate: datetime | None = Field(default=None, alias="Modified")
    emailsSent: int | None = Field(default=None, alias="EmailsSent")
    participants: int | None = Field(default=None, alias="Participants")
    emailsBounced: int | None = Field(default=None, alias="EmailsBounced")
    numberOfClickedParticipants: int | None = Field(default=None, alias="NumberOfClickedParticipants")
    numberOfTrainedParticipants: int | None = Field(default=None, alias="NumberOfTrainedParticipants")
    numberOfTrainedParticipatns: int | None = Field(default=None, alias="NumberOfTrainedParticipatns") #Seems ironscales had a type and for some reason continued to include it alongside the fixed string
    numberOfReportedParticipants: int | None = Field(default=None, alias="NumberOfReportedParticipants")
    numberOfReadParticipants: int | None = Field(default=None, alias="NumberOfReadParticipants")
    numberOfDeleted: int | None = Field(default=None, alias="NumberOfDeleted")
    attackReadinessFirstReport: str | None = Field(default=None, alias="AttackReadinessFirstReport")
    attackReadinessMitigationTime: str | None = Field(default=None, alias="AttackReadinessMitigationTime")
    attackReadinessLuredBeforeMitigation: str | None = Field(default=None, alias="AttackReadinessLuredBeforeMitigation")
    attackReadinessReportsToMitigate: str | None = Field(default=None, alias="AttackReadinessReportsToMitigate")
    companyId: int | None = Field(default=None, alias="CompanyId")
    randomized: bool | None = Field(default=None, alias="Randomized")

class CampaignParticipants(IronscalesModel):
    internalID: int | None = Field(default=None, alias="InternalID")
    name: str | None = Field(default=None, alias="Name")
    displayName: str | None = Field(default=None, alias="DisplayName")
    lastUpdate: datetime | None = Field(default=None, alias="LastUpdate")
    title: str | None = Field(default=None, alias="Title")
    department: str | None = Field(default=None, alias="Department")
    company: str | None = Field(default=None, alias="Company")
    manager: str | None = Field(default=None, alias="Manager")
    office: str | None = Field(default=None, alias="Office")
    country: str | None = Field(default=None, alias="Country")
    city: str | None = Field(default=None, alias="City")
    sentAt: datetime | None = Field(default=None, alias="SentAt")
    opened: str | None = Field(default=None, alias="Opened")
    openedAt: datetime | None = Field(default=None, alias="OpenedAt")
    enteredDetails: str | None = Field(default=None, alias="EnteredDetails")
    trainingModule: str | None = Field(default=None, alias="TrainingModule")
    trainingVideoStarted: str | None = Field(default=None, alias="TrainingVideoStarted")
    awarenessLevel: str | None = Field(default=None, alias="AwarenessLevel")
    customTags: str | None = Field(default=None, alias="CustomTags")
    reported: str | None = Field(default=None, alias="Reported")
    reportedTime: datetime | None = Field(default=None, alias="ReportedTime")
    read: str | None = Field(default=None, alias="Read")
    clicked: str | None = Field(default=None, alias="Clicked")
    clickedTime: datetime | None = Field(default=None, alias="ClickedTime")
    resendTrainingDates: list[datetime] | None = Field(default=None, alias="ResendTrainingDates")
    deleted: str | None = Field(default=None, alias="Deleted")
    trained: str | None = Field(default=None, alias="Trained")
    trainingCompletionDate: datetime | None = Field(default=None, alias="TrainingCompletionDate")
    trainingScore: int | None = Field(default=None, alias="TrainingScore")
    trainingDuration: int | None = Field(default=None, alias="TrainingDuration")
    trainingStartedOn: datetime | None = Field(default=None, alias="TrainingStartedOn")
    email: str | None = Field(default=None, alias="Email")
    template: str | None = Field(default=None, alias="Template")
    userIp: str | None = Field(default=None, alias="UserIp")

class PartnerCompany(IronscalesModel):
    id: int | None = Field(default=None, alias="Id")
    name: str | None = Field(default=None, alias="Name")
    domain: str | None = Field(default=None, alias="Domain")
    ownerEmail: str | None = Field(default=None, alias="OwnerEmail")
    ownerName: str | None = Field(default=None, alias="OwnerName")
    country: str | None = Field(default=None, alias="Country")
    registrationDate: datetime | None = Field(default=None, alias="RegistrationDate")
    partner_id: int | None = Field(default=None, alias="PartnerID")

class PartnerCompanyV2(IronscalesModel):
    id: int | None = Field(default=None, alias="Id")
    name: str | None = Field(default=None, alias="Name")
    domain: str | None = Field(default=None, alias="Domain")
    ownerEmail: str | None = Field(default=None, alias="OwnerEmail")
    ownerName: str | None = Field(default=None, alias="OwnerName")
    country: str | None = Field(default=None, alias="Country")
    registrationDate: datetime | None = Field(default=None, alias="RegistrationDate")
    partner_id: int | None = Field(default=None, alias="PartnerID")
    planExpirationDate: datetime | None = Field(default=None, alias="PlanExpirationDate")
    trialPlanExpirationDate: datetime | None = Field(default=None, alias="TrialPlanExpirationDate")
    
class CompanyAutoSyncStatus(IronscalesModel):
    in_progress: bool | None = Field(default=None, alias="InProgress")
    mailboxes_total_count: int | None = Field(default=None, alias="MailboxesTotalCount")
    protected_mailboxes_count: int | None = Field(default=None, alias="ProtectedMailboxesCount")
    enabled_mailboxes_count: int | None = Field(default=None, alias="EnabledMailboxesCount")
    synced_mailboxes_count: int | None = Field(default=None, alias="SyncedMailboxesCount")
    failed_mailboxes_count: int | None = Field(default=None, alias="FailedMailboxesCount")
    last_synced_at: datetime | None = Field(default=None, alias="LastSyncedAt")

class Company911Email(IronscalesModel):
    email: str | None = Field(default=None, alias="Email")

class CompanyAutoSyncGroups(IronscalesModel):
    id: str | None = Field(default=None, alias="Id")
    display_name: str | None = Field(default=None, alias="DisplayName")
    
class CompanyManifest(IronscalesModel):
    report_button: str | None = Field(default=None, alias="ReportButton")
    add_in_description: str | None = Field(default=None, alias="AddInDescription")
    report_phishing_caption: str | None = Field(default=None, alias="ReportPhishingCaption")
    provider_name: str | None = Field(default=None, alias="ProviderName")
    logo: str | None = Field(default=None, alias="Logo")
    
class CompanySyncedEmails(IronscalesModel):
    first_name: str | None = Field(default=None, alias="FirstName")
    last_name: str | None = Field(default=None, alias="LastName")
    email: str | None = Field(default=None, alias="Email")
    change_date: datetime | None = Field(default=None, alias="ChangeDate")
    
class CompanyFeaturesStates(IronscalesModel):
    silentMode: bool | None = Field(default=None, alias="SilentMode")
    silentModeMsg: bool | None = Field(default=None, alias="SilentModeMsg")
    ato: bool | None = Field(default=None, alias="ATO")
    serviceManagement: bool | None = Field(default=None, alias="ServiceManagement")
    trainingCampaignsWizer: bool | None = Field(default=None, alias="TrainingCampaignsWizer")
    api: bool | None = Field(default=None, alias="API")
    themisCoPilot: bool | None = Field(default=None, alias="ThemisCoPilot")
    attachmentsScan: bool | None = Field(default=None, alias="AttachmentsScan")
    linksScan: bool | None = Field(default=None, alias="LinksScan")
    STbundle: bool | None = Field(default=None, alias="STBundle")
    SATBundlePlus: bool | None = Field(default=None, alias="SATBundlePlus")
    AiEmpowerBundle: bool | None = Field(default=None, alias="AiEmpowerBundle")
    autopilotEnabled: bool | None = Field(default=None, alias="AutopilotEnabled")

class CompanyStatisticsAndLicense(IronscalesModel):
    openIncidentCount: int | None = Field(default=None, alias="OpenIncidentCount")
    highPriorityIncidentCount: int | None = Field(default=None, alias="highPriorityIncidentCount")
    mediumPriorityIncidentCount: int | None = Field(default=None, alias="mediumPriorityIncidentCount")
    lowPriorityIncidentCount: int | None = Field(default=None, alias="lowPriorityIncidentCount")
    activeAttacksCount: int | None = Field(default=None, alias="activeAttacksCount")
    license: dict[str, Any] | None = Field(default=None, alias="license")
    protectedMailboxes: bool | None = Field(default=None, alias="protectedMailboxes")
    activeMailboxes: bool | None = Field(default=None, alias="activeMailboxes")
    lastMailboxSyncDate: datetime | None = Field(default=None, alias="lastMailboxSyncDate")

class EscalatedEmails(IronscalesModel):
    arrival_date: datetime | None = Field(default=None, alias="ArrivalDate")
    incident_id: int | None = Field(default=None, alias="IncidentId")
    subject: str | None = Field(default=None, alias="Subject")
    sender_email: str | None = Field(default=None, alias="SenderEmail")
    recipient_name: str | None = Field(default=None, alias="RecipientName")
    recipient_email: str | None = Field(default=None, alias="RecipientEmail")
    primary_threat_type: str | None = Field(default=None, alias="PrimaryThreatType")
    is_scanback: bool | None = Field(default=None, alias="IsScanback")
    classification: str | None = Field(default=None, alias="Classification")
    incident_state: str | None = Field(default=None, alias="IncidentState")
    resolution: str | None = Field(default=None, alias="Resolution")
    sender_ip: str | None = Field(default=None, alias="SenderIp")
    reported_by: str | None = Field(default=None, alias="ReportedBy")
    mailbox_id: int | None = Field(default=None, alias="MailboxId")
    department: str | None = Field(default=None, alias="Department")
    remediated_time: datetime | None = Field(default=None, alias="RemediatedTime")
    mitigation_id: int | None = Field(default=None, alias="MitigationId")

class IncidentDetails(IronscalesModel):
    company_id: int | None = Field(default=None, alias="CompanyId")
    company_name: str | None = Field(default=None, alias="CompanyName")
    incident_id: int | None = Field(default=None, alias="IncidentId")
    classification: str | None = Field(default=None, alias="Classification")
    first_reported_by: str | None = Field(default=None, alias="FirstReportedBy")
    first_reported_date: datetime | None = Field(default=None, alias="FirstReportedDate")
    affected_mailbox_count: int | None = Field(default=None, alias="AffectedMailboxCount")
    sender_reputation: str | None = Field(default=None, alias="SenderReputation")
    banner_displayed: bool | None = Field(default=None, alias="BannerDisplayed")
    sender_email: str | None = Field(default=None, alias="SenderEmail")
    reply_to: str | None = Field(default=None, alias="ReplyTo")
    spf_result: str | None = Field(default=None, alias="SPFResult")
    sender_is_internal: bool | None = Field(default=None, alias="SenderIsInternal")
    themis_proba: float | None = Field(default=None, alias="ThemisProba")
    themis_verdict: str | None = Field(default=None, alias="ThemisVerdict")
    mail_server: dict[str, str] | None = Field(default=None, alias="MailServer")
    federation: dict[str, int | float] | None = Field(default=None, alias="Federation")
    reports: dict[str, str | dict[str, str]] | None = Field(default=None, alias="Reports")
    links: dict[str, str] | None = Field(default=None, alias="Links")
    attachments: dict[str, str | int] | None = Field(default=None, alias="Attachments")
    original_email_body: str | None = Field(default=None, alias="OriginalEmailBody")
    email_body_text: str | None = Field(default=None, alias="EmailBodyText")
    reported_by_end_user: bool | None = Field(default=None, alias="ReportedByEndUser")
    reporter_name: str | None = Field(default=None, alias="ReporterName")

class Incidents(IronscalesModel):
    incidentID: int | None = Field(default=None, alias="ArrivalDate")
    emailSubject: str | None = Field(default=None, alias="EmailSubject")
    linksCount: int | None = Field(default=None, alias="LinksCount")
    attachmentsCount: int | None = Field(default=None, alias="AttachmentsCount")
    recipientEmail: str | None = Field(default=None, alias="RecipientEmail")
    recipientName: str | None = Field(default=None, alias="RecipientName")
    classification: str | None = Field(default=None, alias="Classification")
    assignee: str | None = Field(default=None, alias="Assignee")
    senderName: str | None = Field(default=None, alias="SenderName")
    senderEmail: str | None = Field(default=None, alias="SenderEmail")
    affectedMailboxesCount: int | None = Field(default=None, alias="AffectedMailboxesCount")
    created: str | None = Field(default=None, alias="Created")
    reportedBy: str | None = Field(default=None, alias="ReportedBy")
    resolvedBy: str | None = Field(default=None, alias="ResolvedBy")
    incidentType: str | None = Field(default=None, alias="IncidentsType")
    commentsCount: int | None = Field(default=None, alias="CommentsCount")
    releaseRequestCount: int | None = Field(default=None, alias="ReleaseRequestCount")
    latestEmailDate: datetime | None = Field(default=None, alias="LatestEmailDate")

class IncidentClassify(IronscalesModel):
    classification: int | None = Field(default=None, alias="Classification")
    prev_classification: str | None = Field(default=None, alias="PrevClassification")
    classifying_user_email: int | None = Field(default=None, alias="ClassifyingUserEmail")

class ScanbackIncidents(IronscalesModel):
    incidentID: int | None = Field(default=None, alias="ArrivalDate")
    emailSubject: str | None = Field(default=None, alias="EmailSubject")
    linksCount: int | None = Field(default=None, alias="LinksCount")
    attachmentsCount: int | None = Field(default=None, alias="AttachmentsCount")
    recipientEmail: str | None = Field(default=None, alias="RecipientEmail")
    recipientName: str | None = Field(default=None, alias="RecipientName")
    classification: str | None = Field(default=None, alias="Classification")
    assignee: str | None = Field(default=None, alias="Assignee")
    senderName: str | None = Field(default=None, alias="SenderName")
    senderEmail: str | None = Field(default=None, alias="SenderEmail")
    affectedMailboxesCount: int | None = Field(default=None, alias="AffectedMailboxesCount")
    created: str | None = Field(default=None, alias="Created")
    reportedBy: str | None = Field(default=None, alias="ReportedBy")
    resolvedBy: str | None = Field(default=None, alias="ResolvedBy")

class RemediationStatusesStats(IronscalesModel):
    phishing: dict[str, int] | None = Field(default=None, alias="Phishing")
    spam: dict[str, int] | None = Field(default=None, alias="Spam")
    safe: dict[str, int] | None = Field(default=None, alias="Safe")
    unclassified: dict[str, int] | None = Field(default=None, alias="Unclassified")

class UnclassifiedIncidentIDs(IronscalesModel):
    incident_ids: list[int] | None = Field(default=None, alias="IncidentIds")

class IntegrationStatus(IronscalesModel):
    company_id: int | None = Field(default=None, alias="CompanyId")
    integration_type: str | None = Field(default=None, alias="IntegrationType")
    is_integrated: bool | None = Field(default=None, alias="IsIntegrated")

class IntegrationO365Authorize(IronscalesModel):
    admin_consent: bool | None = Field(default=None, alias="AdminConsent")
    state: str | None = Field(default=None, alias="State")
    tenant: str | None = Field(default=None, alias="Tenant")
    error: str | None = Field(default=None, alias="Error")
    error_description: str | None = Field(default=None, alias="ErrorDescription")

class IntegrationO365AuthorizeResponse(IronscalesModel):
    additional_data: Any | None = Field(default=None, alias="AdditionalData")
    
class IntegrationGWSAuthorizeResponse(IronscalesModel):
    error_message: Any | None = Field(default=None, alias="AdditionalData")
    
class IntegrationDisableIntegration(IronscalesModel):
    company_id: int | None = Field(default=None, alias="CompanyId")
    integration_type: str | None = Field(default=None, alias="IntegrationType")
    integration_status: str | None = Field(default=None, alias="IntegrationStatus")
    
class IntegrationsGWSConsentRedirectURL(IronscalesModel):
    oauth_full_url: str | None = Field(default=None, alias="OAuthFullUrl")

class IntegrationsO365ConsentRedirectURL(IronscalesModel):
    azure_redirect_uri: str | None = Field(default=None, alias="AzureRedirectURI")
    additional_data: Any | None = Field(default=None, alias="AdditionalData")

class IntegrationsO365ConsentRedirectURLResponse(IronscalesModel):
    oauth_full_url: str | None = Field(default=None, alias="OAuthFullUrl")

class ComplianceReport(IronscalesModel):
    id: int | None = Field(default=None, alias="Id")
    firstName: str | None = Field(default=None, alias="FirstName")
    lastName: str | None = Field(default=None, alias="LastName")
    country: str | None = Field(default=None, alias="Country")
    department: str | None = Field(default=None, alias="Department")
    title: str | None = Field(default=None, alias="Title")
    email: str | None = Field(default=None, alias="Email")
    language: str | None = Field(default=None, alias="Language")
    simulationCampaignsCompletionsCount: int | None = Field(default=None, alias="SimulationCampaignsCompletionsCount")
    lastSimulationCampaignDate: datetime | None = Field(default=None, alias="LastSimulationCampaignDate")
    trainingCampaignsCompletionsCount: int | None = Field(default=None, alias="TrainingCampaignsCompletionCount")
    lastTrainingCampaignDate: datetime | None = Field(default=None, alias="LastTrainingCampaignDate")
    riskLevel: str | None = Field(default=None, alias="RiskLevel")
    awarenessLevel: str | None = Field(default=None, alias="AwarenessLevel")

class CompanyMailboxes(IronscalesModel):
    id: int | None = Field(default=None, alias="Id")
    firstName: str | None = Field(default=None, alias="FirstName")
    lastName: str | None = Field(default=None, alias="LastName")
    title: str | None = Field(default=None, alias="Title")
    department: str | None = Field(default=None, alias="Department")
    email: str | None = Field(default=None, alias="Email")
    phoneNumber: str | None = Field(default=None, alias="PhoneNumber")
    tags: list[str] | None = Field(default=None, alias="Tags")
    language: str | None = Field(default=None, alias="Language")
    enabled: bool | None = Field(default=None, alias="Enabled")
    riskLevel: str | None = Field(default=None, alias="RiskLevel")
    awarenessLevel: str | None = Field(default=None, alias="AwarenessLevel")
    protected: bool | None = Field(default=None, alias="Protected")
    unprotectedReason: str | None = Field(default=None, alias="UnprotectedReason")

class CompanyMailboxesPutResponse(IronscalesModel):
    mailbox_ids: list[int] | None = Field(default=None, alias="MailboxIds")
    error_message: str | None = Field(default=None, alias="ErrorMessage")

class UserCampaignPerformance(IronscalesModel):
    id: int | None = Field(default=None, alias="Id")
    userId: int | None = Field(default=None, alias="UserId")
    firstName: str | None = Field(default=None, alias="FirstName")
    lastName: str | None = Field(default=None, alias="LastName")
    country: str | None = Field(default=None, alias="Country")
    department: str | None = Field(default=None, alias="Department")
    title: str | None = Field(default=None, alias="Title")
    email: str | None = Field(default=None, alias="Email")
    campaignId: int | None = Field(default=None, alias="CampaignId")
    campaignName: str | None = Field(default=None, alias="CampaignName")
    campaignType: str | None = Field(default=None, alias="CampaignType")
    campaignSimulationResult: str | None = Field(default=None, alias="CampaignSimulationResult")
    campaignTrainingStatus: str | None = Field(default=None, alias="CampaignTrainingStatus")
    campaignTrainingName: str | None = Field(default=None, alias="CampaignTrainingName")
    campaignCollectingEndDate: datetime | None = Field(default=None, alias="CampaignCollectingEndDate")
    campaignScore: int | None = Field(default=None, alias="CampaignScore")
    campaignLocale: str | None = Field(default=None, alias="CampaignLocale")
    campaignTemplateName: str | None = Field(default=None, alias="CampaignTemplateName")

class CompanyImpersonationIncidents(IronscalesModel):
    incidentID: int | None = Field(default=None, alias="IncidentId")
    mailboxId: int | None = Field(default=None, alias="MailboxId")
    remediatedTime: datetime | None = Field(default=None, alias="RemediatedTime")
    mailboxEmail: str | None = Field(default=None, alias="MailboxEmail")
    senderEmail: str | None = Field(default=None, alias="SenderEmail")
    subject: str | None = Field(default=None, alias="Subject")
    reportedBy: str | None = Field(default=None, alias="ReportedBy")
    incidentType: str | None = Field(default=None, alias="IncidentType")
    resolution: str | None = Field(default=None, alias="Resolution")
    remediations: int | None = Field(default=None, alias="Remediations")

class CompanyMitigationDetails(IronscalesModel):
    incidentID: int | None = Field(default=None, alias="IncidentId")
    incidentState: int | None = Field(default=None, alias="IncidentState")
    remediatedTime: datetime | None = Field(default=None, alias="RemediatedTime")
    affectedMailboxCount: int | None = Field(default=None, alias="AffectedMailboxCount")
    mailboxId: str | None = Field(default=None, alias="MailboxId")
    mailboxEmail: str | None = Field(default=None, alias="MailboxEmail")
    senderEmail: str | None = Field(default=None, alias="SenderEmail")
    subject: str | None = Field(default=None, alias="Subject")
    threatType: str | None = Field(default=None, alias="ThreatType")
    detectionType: str | None = Field(default=None, alias="DetectionType")
    reportedBy: str | None = Field(default=None, alias="ReportedBy")

class CompanyMitigationDetailsPostResponse(IronscalesModel):
    incidentID: int | None = Field(default=None, alias="IncidentId")
    mitigationID: int | None = Field(default=None, alias="MitigationID")
    incidentState: int | None = Field(default=None, alias="IncidentState")
    remediatedTime: datetime | None = Field(default=None, alias="RemediatedTime")
    mailboxId: str | None = Field(default=None, alias="MailboxId")
    mailboxEmail: str | None = Field(default=None, alias="MailboxEmail")
    subject: str | None = Field(default=None, alias="Subject")
    senderEmail: str | None = Field(default=None, alias="SenderEmail")
    senderIP: str | None = Field(default=None, alias="SenderIP")
    reportedBy: str | None = Field(default=None, alias="ReportedBy")
    resolution: str | None = Field(default=None, alias="Resolution")
    spfResult: str | None = Field(default=None, alias="SPFResult")

class CompanyMitigationStatistics(IronscalesModel):
    openIncidentCount: int | None = Field(default=None, alias="OpenIncidentCount")
    resolvedIncidentCount: int | None = Field(default=None, alias="ResolvedIncidentCount")
    phishingCount: int | None = Field(default=None, alias="PhishingCount")
    remediationCount: int | None = Field(default=None, alias="RemediationCount")
    maliciousAttachmentsCount: int | None = Field(default=None, alias="MaliciousAttachmentsCount")
    maliciousLinksCount: int | None = Field(default=None, alias="MaliciousLinksCount")
    impersonationCount: int | None = Field(default=None, alias="ImpersonationCount")
    reportedByEmployeesCount: int | None = Field(default=None, alias="ReportedByEmployeesCount")

class CompanyEmailStatistics(IronscalesModel):
    inspected_count: int | None = Field(default=None, alias="InspectedCount")
    phishing_count: int | None = Field(default=None, alias="PhishingCount")
    spam_count: int | None = Field(default=None, alias="SpamCount")
    impersonations_count: int | None = Field(default=None, alias="ImpersonationsCount")
    phishing_threat_types: dict[str, int] | None = Field(default=None, alias="PhishingThreatTypes")

class MostTargetedDepartments(IronscalesModel):
    name: int | None = Field(default=None, alias="Name")
    emails_count: int | None = Field(default=None, alias="EmailsCount")

class MostTargetedEmployees(IronscalesModel):
    mailbox: int | None = Field(default=None, alias="Mailbox")
    emails_count: int | None = Field(default=None, alias="EmailsCount")

class CompanyMitigationStatisticsV2(IronscalesModel):
    resolved_by_analyst: dict[str, int | float] | None = Field(default=None, alias="ResolvedByAnalyst")
    inspected_emails: dict[str, int] | None = Field(default=None, alias="InspectedEmails")
    resolved_automatically: dict[str, int | float] | None = Field(default=None, alias="ResolvedAutomatically")
    malicious_content_incidents: dict[str, int] | None = Field(default=None, alias="MaliciousContectIncidents")

class CompanyLicensedDomains(IronscalesModel):
    company_id: int | None = Field(default=None, alias="CompanyId")
    licensed_domains: list[str] | None = Field(default=None, alias="LicensedDomains")

class CompanyLicensedDomainsPutResponse(IronscalesModel):
    company_id: int | None = Field(default=None, alias="CompanyId")
    domains_added: list[str] | None = Field(default=None, alias="DomainsAdded")
    existing_domains: list[str] | None = Field(default=None, alias="ExistingDomains")

class CompanyLicense(IronscalesModel):
    id: int | None = Field(default=None, alias="Id")
    trialExpiration: datetime | None = Field(default=None, alias="TrialExpiration")
    trialPlanType: str | None = Field(default=None, alias="TrialPlanType")
    premiumContentType: str | None = Field(default=None, alias="PremiumContentType")
    planType: str | None = Field(default=None, alias="PlanType")
    planExpiration: str | None = Field(default=None, alias="PlanExpiration")
    mailboxLimit: int | None = Field(default=None, alias="MailboxLimit")
    is_partner: bool | None = Field(default=None, alias="IsPartner")
    testMode: bool | None = Field(default=None, alias="TestMode")

class AccountTakeoverSensitivySettings(IronscalesModel):
    sensitivity: int | None = Field(default=None, alias="Sensitivity")

class AllowListSettings(IronscalesModel):
    allow_list: dict[str, Any] | None = Field(default=None, alias="AllowList")
    internal_active: bool | None = Field(default=None, alias="InternalActive")
    external_active: bool | None = Field(default=None, alias="ExternalActive")

class ChallengedAlerts(IronscalesModel):
    recipients: list[str] | None = Field(default=None, alias="Recipients")

class CompanyNotificationSettings(IronscalesModel):
    recipients: list[str] | None = Field(default=None, alias="Recipients")




from pyironscales.endpoints.base.base_endpoint import IronscalesEndpoint
from pyironscales.endpoints.ironscales.MailboxesIdComplianceReportEndpoint import MailboxesIdComplianceReportEndpoint
from pyironscales.endpoints.ironscales.MailboxesIdListEndpoint import MailboxesIdListEndpoint
from pyironscales.endpoints.ironscales.MailboxesIdUserCampaignsPerformanceEndpoint import MailboxesIdUserCampaignsPerformanceEndpoint

class MailboxesIdEndpoint(
    IronscalesEndpoint,
):
    def __init__(self, client, parent_endpoint=None) -> None:
        IronscalesEndpoint.__init__(self, client, "{id}", parent_endpoint=parent_endpoint)
        self.compliance_report = self._register_child_endpoint(MailboxesIdComplianceReportEndpoint(client, parent_endpoint=self))
        self.list = self._register_child_endpoint(MailboxesIdListEndpoint(client, parent_endpoint=self))
        self.user_campaigns_performance = self._register_child_endpoint(MailboxesIdUserCampaignsPerformanceEndpoint(client, parent_endpoint=self))



from pyironscales.endpoints.base.base_endpoint import IronscalesEndpoint
from pyironscales.models.ironscales import Answer
from pyironscales.endpoints.ironscales.CampaignsIdDetailsEndpoint import CampaignsIdDetailsEndpoint
from pyironscales.endpoints.ironscales.CampaignsIdParticipantDetailsEndpoint import CampaignsIdParticipantDetailsEndpoint
from pyironscales.types import (
    JSON,
    IronscalesRequestParams,
)


class CampaignsIdEndpoint(
    IronscalesEndpoint,
):
    def __init__(self, client, parent_endpoint=None) -> None:
        IronscalesEndpoint.__init__(self, client, "{id}", parent_endpoint=parent_endpoint)
        self.search = self._register_child_endpoint(CampaignsIdDetailsEndpoint(client, parent_endpoint=self))
        self.search = self._register_child_endpoint(CampaignsIdParticipantDetailsEndpoint(client, parent_endpoint=self))



from pyironscales.endpoints.base.base_endpoint import IronscalesEndpoint
from pyironscales.interfaces import (
    IPostable,
)
from pyironscales.models.ironscales import IncidentClassify
from pyironscales.types import (
    JSON,
    IronscalesRequestParams,
)


class IncidentIdClassifyIdEndpoint(
    IronscalesEndpoint,
    IPostable[IncidentClassify, IronscalesRequestParams],
):
    def __init__(self, client, parent_endpoint=None) -> None:
        IronscalesEndpoint.__init__(self, client, "{id}", parent_endpoint=parent_endpoint)
        IPostable.__init__(self, IncidentClassify)

    def post(self, data: JSON | None = None, params: IronscalesRequestParams | None = None) -> IncidentClassify:
        """
        Performs a POST request against the /incident/{id}/classify/{id} endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            IncidentClassify: The parsed Company data.
        """
        return self._parse_one(IncidentClassify, super()._make_request("POST", data=data, params=params).json())



from pyironscales.endpoints.base.base_endpoint import IronscalesEndpoint
from pyironscales.endpoints.ironscales.IncidentIdClassifyEndpoint import IncidentIdClassifyEndpoint
from pyironscales.endpoints.ironscales.IncidentIdDetailsEndpoint import IncidentIdDetailsEndpoint
from pyironscales.endpoints.ironscales.IncidentIdListEndpoint import IncidentIdListEndpoint
from pyironscales.endpoints.ironscales.IncidentIdScanbackListEndpoint import IncidentIdScanbackListEndpoint
from pyironscales.endpoints.ironscales.IncidentIdStatsEndpoint import IncidentIdStatsEndpoint
from pyironscales.endpoints.ironscales.IncidentIdStatusEndpoint import IncidentIdStatusEndpoint

class IncidentIdEndpoint(
    IronscalesEndpoint,
):
    def __init__(self, client, parent_endpoint=None) -> None:
        IronscalesEndpoint.__init__(self, client, "{id}", parent_endpoint=parent_endpoint)
        self.classify = self._register_child_endpoint(IncidentIdClassifyEndpoint(client, parent_endpoint=self))
        self.details = self._register_child_endpoint(IncidentIdDetailsEndpoint(client, parent_endpoint=self))
        self.list = self._register_child_endpoint(IncidentIdListEndpoint(client, parent_endpoint=self))
        self.scanback_list = self._register_child_endpoint(IncidentIdScanbackListEndpoint(client, parent_endpoint=self))
        self.stats = self._register_child_endpoint(IncidentIdStatsEndpoint(client, parent_endpoint=self))

    def id(self, id: int) -> IncidentIdStatusEndpoint:
        """
        Sets the ID for this endpoint and returns an initialized IncidentIdStatusEndpoint object to move down the chain.

        Parameters:
            id (int): The ID to set.
        Returns:
            IncidentIdStatusEndpoint: The initialized IncidentIdStatusEndpoint object.
        """
        child = IncidentIdStatusEndpoint(self.client, parent_endpoint=self)
        child._id = id
        return child



from pyironscales.endpoints.base.base_endpoint import IronscalesEndpoint
from pyironscales.interfaces import (
    IGettable,
    IPaginateable,
)
from pyironscales.models.ironscales import IncidentDetails
from pyironscales.responses.paginated_response import PaginatedResponse
from pyironscales.types import (
    JSON,
    IronscalesRequestParams,
)

class MitigationsIdIncidentsDetailsEndpoint(
    IronscalesEndpoint,
    IGettable[IncidentDetails, IronscalesRequestParams],
    IPaginateable[IncidentDetails, IronscalesRequestParams],
):
    def __init__(self, client, parent_endpoint=None) -> None:
        IronscalesEndpoint.__init__(self, client, "details/", parent_endpoint=parent_endpoint)
        IGettable.__init__(self, IncidentDetails)
        IPaginateable.__init__(self, IncidentDetails)

    def paginated(
        self,
        page: int,
        params: IronscalesRequestParams | None = None,
    ) -> PaginatedResponse[IncidentDetails]:
        """
        Performs a GET request against the /mitigations/{id}/incidents/details/ endpoint and returns an initialized PaginatedResponse object.

        Parameters:
            page (int): The page number to request.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            PaginatedResponse[IncidentDetails]: The initialized PaginatedResponse object.
        """
        if params:
            params["page"] = page
        else:
            params = {"page": page}
        return PaginatedResponse(
            super()._make_request("GET", params=params),
            IncidentDetails,
            self,
            "incidents",
            page,
            params,
        )

    def get(
        self,
        data: JSON | None = None,
        params: IronscalesRequestParams | None = None,
    ) -> IncidentDetails:
        """
        Performs a GET request against the /mitigations/{id}/incidents/details/ endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            IncidentDetails: The parsed response data.
        """
        return self._parse_many(
            IncidentDetails,
            super()._make_request("GET", data=data, params=params).json().get('incidents', {}),
        )



from pyironscales.endpoints.base.base_endpoint import IronscalesEndpoint
from pyironscales.endpoints.ironscales.IntegrationsIdIntegrationStatusEndpoint import IntegrationsIdIntegrationStatusEndpoint
from pyironscales.endpoints.ironscales.IntegrationsIdDisableIntegrationEndpoint import IntegrationsIdDisableIntegrationEndpoint
from pyironscales.endpoints.ironscales.IntegrationsIdGWSAuthorizeEndpoint import IntegrationsIdGWSAuthorizeEndpoint
from pyironscales.endpoints.ironscales.IntegrationsIdGWSConsentRedirectURIEndpoint import IntegrationsIdGWSConsentRedirectURIEndpoint
from pyironscales.endpoints.ironscales.IntegrationsIdO365AuthorizeEndpoint import IntegrationsIdO365AuthorizeEndpoint
from pyironscales.endpoints.ironscales.IntegrationsIdO365ConsentRedirectURIEndpoint import IntegrationsIdO365ConsentRedirectURIEndpoint

class IntegrationsIdEndpoint(
    IronscalesEndpoint,
):
    def __init__(self, client, parent_endpoint=None) -> None:
        IronscalesEndpoint.__init__(self, client, "{id}", parent_endpoint=parent_endpoint)
        self.integration_status = self._register_child_endpoint(IntegrationsIdIntegrationStatusEndpoint(client, parent_endpoint=self))
        self.disable_integration = self._register_child_endpoint(IntegrationsIdDisableIntegrationEndpoint(client, parent_endpoint=self))
        self.gws_authorize_endpoint = self._register_child_endpoint(IntegrationsIdGWSAuthorizeEndpoint(client, parent_endpoint=self))
        self.gws_consent_redirect_uri = self._register_child_endpoint(IntegrationsIdGWSConsentRedirectURIEndpoint(client, parent_endpoint=self))
        self.o365_authorize = self._register_child_endpoint(IntegrationsIdO365AuthorizeEndpoint(client, parent_endpoint=self))
        self.o365_consent_redirect_uri = self._register_child_endpoint(IntegrationsIdO365ConsentRedirectURIEndpoint(client, parent_endpoint=self))



from pyironscales.endpoints.base.base_endpoint import IronscalesEndpoint
from pyironscales.interfaces import (
    IGettable,
    IPostable,
    IPuttable,
    IDeleteable,
)
from pyironscales.models.ironscales import ChallengedAlerts
from pyironscales.types import (
    JSON,
    IronscalesRequestParams,
)

class SettingsIdChallengedAlertsEndpoint(
    IronscalesEndpoint,
    IGettable[ChallengedAlerts, IronscalesRequestParams],
    IPostable[ChallengedAlerts, IronscalesRequestParams],
    IPuttable[ChallengedAlerts, IronscalesRequestParams],
    IDeleteable[IronscalesRequestParams],
):
    def __init__(self, client, parent_endpoint=None) -> None:
        IronscalesEndpoint.__init__(self, client, "challenged-alerts/", parent_endpoint=parent_endpoint)
        IGettable.__init__(self, ChallengedAlerts)
        IPuttable.__init__(self, ChallengedAlerts)

    def get(
        self,
        data: JSON | None = None,
        params: IronscalesRequestParams | None = None,
    ) -> ChallengedAlerts:
        """
        Performs a GET request against the /settings/{id}/challenged-alerts/ endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            ChallengedAlerts: The parsed response data.
        """
        return self._parse_one(
            ChallengedAlerts,
            super()._make_request("GET", data=data, params=params).json(),
        )

    def post(self, data: JSON | None = None, params: IronscalesRequestParams | None = None) -> ChallengedAlerts:
        """
        Performs a POST request against the /settings/{id}/challenged-alerts/ endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            ChallengedAlerts: The parsed Company data.
        """
        return self._parse_one(ChallengedAlerts, super()._make_request("POST", data=data, params=params).json())

    def put(
        self,
        data: JSON | None = None,
        params: IronscalesRequestParams | None = None,
    ) -> ChallengedAlerts:
        """
        Performs a PUT request against the /settings/{id}/challenged-alerts/ endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            ChallengedAlerts: The parsed response data.
        """
        return self._parse_one(
            ChallengedAlerts,
            super()._make_request("PUT", data=data, params=params).json(),
        )

    def delete(self, data: JSON | None = None, params: IronscalesRequestParams | None = None) -> ChallengedAlerts:
        """
        Performs a DELETE request against the /settings/{id}/challenged-alerts/ endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            ChallengedAlerts: The parsed response data.
        """
        return self._parse_one(ChallengedAlerts, super()._make_request("DELETE", data=data, params=params).json())



from pyironscales.endpoints.base.base_endpoint import IronscalesEndpoint
from pyironscales.endpoints.ironscales.CompanyListV2Endpoint import CompanyListV2Endpoint

from pyironscales.interfaces import (
    IGettable,
)
from pyironscales.models.ironscales import PartnerCompany
from pyironscales.types import (
    JSON,
    IronscalesRequestParams,
)


class CompanyListEndpoint(
    IronscalesEndpoint,
    IGettable[PartnerCompany, IronscalesRequestParams],
):
    def __init__(self, client, parent_endpoint=None) -> None:
        IronscalesEndpoint.__init__(self, client, "list", parent_endpoint=parent_endpoint)
        IGettable.__init__(self, PartnerCompany)
        self.v2 = self._register_child_endpoint(CompanyListV2Endpoint(client, parent_endpoint=self))

    def get(
        self,
        data: JSON | None = None,
        params: IronscalesRequestParams | None = None,
    ) -> PartnerCompany:
        """
        Performs a GET request against the /company/list endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            AuthInformation: The parsed response data.
        """
        return self._parse_many(
            PartnerCompany,
            super()._make_request("GET", data=data, params=params).json().get('companies', {}),
        )



from pyironscales.endpoints.base.base_endpoint import IronscalesEndpoint
from pyironscales.endpoints.ironscales.IncidentIdDetailsIdEndpoint import IncidentIdDetailsIdEndpoint


class IncidentIdDetailsEndpoint(
    IronscalesEndpoint,
):
    def __init__(self, client, parent_endpoint=None) -> None:
        IronscalesEndpoint.__init__(self, client, "details", parent_endpoint=parent_endpoint)

    def id(self, id: int) -> IncidentIdDetailsIdEndpoint:
        """
        Sets the ID for this endpoint and returns an initialized IncidentIdDetailsIdEndpoint object to move down the chain.

        Parameters:
            id (int): The ID to set.
        Returns:
            IncidentIdDetailsIdEndpoint: The initialized IncidentIdDetailsIdEndpoint object.
        """
        child = IncidentIdDetailsIdEndpoint(self.client, parent_endpoint=self)
        child._id = id
        return child



from pyironscales.endpoints.base.base_endpoint import IronscalesEndpoint
from pyironscales.interfaces import (
    IPostable,
)
from pyironscales.models.ironscales import CompanyManifest
from pyironscales.types import (
    JSON,
    IronscalesRequestParams,
)


class CompanyIdManifestEndpoint(
    IronscalesEndpoint,
    IPostable[CompanyManifest, IronscalesRequestParams],
):
    def __init__(self, client, parent_endpoint=None) -> None:
        IronscalesEndpoint.__init__(self, client, "manifest/", parent_endpoint=parent_endpoint)
        IPostable.__init__(self, CompanyManifest)

    def post(self, data: JSON | None = None, params: IronscalesRequestParams | None = None) -> CompanyManifest:
        """
        Performs a POST request against the /company/{id}/manifest/ endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            CompanyManifest: The parsed Company data.
        """
        return self._parse_one(CompanyManifest, super()._make_request("POST", data=data, params=params).json())



from pyironscales.endpoints.base.base_endpoint import IronscalesEndpoint
from pyironscales.interfaces import (
    IGettable,
)
from pyironscales.models.ironscales import IntegrationStatus
from pyironscales.types import (
    JSON,
    IronscalesRequestParams,
)


class IntegrationsIdIntegrationStatusEndpoint(
    IronscalesEndpoint,
    IGettable[IntegrationStatus, IronscalesRequestParams],
):
    def __init__(self, client, parent_endpoint=None) -> None:
        IronscalesEndpoint.__init__(self, client, "integration-status/", parent_endpoint=parent_endpoint)
        IGettable.__init__(self, IntegrationStatus)

    def get(
        self,
        data: JSON | None = None,
        params: IronscalesRequestParams | None = None,
    ) -> IntegrationStatus:
        """
        Performs a GET request against the /integrations/{id}/integration-status/ endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            IntegrationStatus: The parsed response data.
        """
        return self._parse_one(
            IntegrationStatus,
            super()._make_request("GET", data=data, params=params).json(),
        )



from pyironscales.endpoints.base.base_endpoint import IronscalesEndpoint
from pyironscales.endpoints.ironscales.MitigationsIdImpersonationDetailsEndpoint import MitigationsIdImpersonationDetailsEndpoint

class MitigationsIdImpersonationEndpoint(
    IronscalesEndpoint,
):
    def __init__(self, client, parent_endpoint=None) -> None:
        IronscalesEndpoint.__init__(self, client, "impersonation", parent_endpoint=parent_endpoint)
        self.details = self._register_child_endpoint(MitigationsIdImpersonationDetailsEndpoint(client, parent_endpoint=self))
